@@ -26,6 +26,7 @@
 #include <libmafw/mafw-log.h>
 #include <libmafw/mafw-registry.h>
 #include <libmafw-shared/mafw-shared.h>
+#include <gconf/gconf-client.h>	
 
 #include "media-player-widget.h"
 
@@ -35,16 +36,32 @@ HD_DEFINE_PLUGIN_MODULE(MediaPlayerWidgetHomePlugin, media_player_widget_home_pl
 #define DEFAULT_WIDGET_WIDTH 256
 #define DEFAULT_WIDGET_HEIGHT 64
 #define MEDIAPLAYER_RENDERER "Mafw-Gst-Renderer"
+#define MEDIAPLAYER_SOURCE "Mafw-Tracker-Source"
+#define PAUSE_ICON "/usr/share/pixmaps/media-player-widget/pause.png"
+#define PLAY_ICON "/usr/share/pixmaps/media-player-widget/play.png"
+#define BACKWARD_ICON "/usr/share/pixmaps/media-player-widget/backward.png"
+#define FORWARD_ICON "/usr/share/pixmaps/media-player-widget/forward.png"
+#define LABEL_BACKGROUND_ICON "/usr/share/pixmaps/media-player-widget/label_background.png"
+#define DEFAULT_ALBUM_ICON "/usr/share/pixmaps/media-player-widget/player.png"
+#define MEDIAPLAYER_LAST_PLAYING_SONG_PATH "/apps/mediaplayer/last_playing_song"
+
+#define HD_APP_MGR_DBUS_NAME "com.nokia.HildonDesktop.AppMgr"
+#define HD_APP_MGR_DBUS_PATH "/com/nokia/HildonDesktop/AppMgr"
+#define HD_APP_MGR_DBUS_IFACE "com.nokia.HildonDesktop.AppMgr"
+#define HD_APP_MGR_LAUNCH_APPLICATION "LaunchApplication"
+#define MEDIAPLAYER_APP_NAME "mediaplayer"
 
 struct _MediaPlayerWidgetHomePluginPrivate
 {
-  gchar* iD;
+  MafwRegistry* mafw_registry;
   MafwRenderer* mafw_renderer;
   MafwPlaylist* mafw_playlist;
+  MafwPlaylistManager* mafw_playlist_manager;
   GtkWidget* play_pause_icon;
   GtkWidget* album_icon;
   GtkWidget* label_background;
   GdkPixbuf* background_pixbuf;
+  GdkPixbuf* album_pixbuf;
   gchar* title;
   gchar* artist;
   gchar* album;
@@ -54,6 +71,8 @@ struct _MediaPlayerWidgetHomePluginPrivate
   gulong on_metadata_changed_handler_id;
   gulong on_playlist_changed_handler_id;
   gulong on_error_handler_id;
+  gulong on_renderer_added_id;
+  gulong on_renderer_removed_id;
 };
 
 
@@ -69,18 +88,40 @@ media_player_widget_expose(GtkWidget* widget, GdkEventExpose* event)
   return GTK_WIDGET_CLASS(media_player_widget_home_plugin_parent_class)->expose_event(widget, event);
 }
 
+static int
+_mediaplayer_last_playlist_index()
+{
+  GConfClient* client = NULL;
+  GError* error = NULL;
+  client = gconf_client_get_default();
+  if(!GCONF_IS_CLIENT(client))
+  {
+    return 0;
+  }
+  int last_playing_song = gconf_client_get_int(client, MEDIAPLAYER_LAST_PLAYING_SONG_PATH, &error);
+  if(error)
+  {
+    g_warning("can not get current view (%s)\n", error->message);
+    g_error_free(error);
+    error = NULL;
+    last_playing_song = 0;
+  }
+  g_object_unref(client);
+  return last_playing_song;
+}
+
 static void
 _set_image_for_state(MafwPlayState state, MediaPlayerWidgetHomePlugin* desktop_plugin)
 {
   if(state == Playing)
   {
     gtk_image_set_from_file(GTK_IMAGE(desktop_plugin->priv->play_pause_icon),
-			    "/usr/share/pixmaps/media-player-widget/pause.png");
+			    PAUSE_ICON);
   }
-  else if(state == Stopped || state == Paused)
+  else if(state == -1 || state == Stopped || state == Paused)
   {
     gtk_image_set_from_file(GTK_IMAGE(desktop_plugin->priv->play_pause_icon),
-			    "/usr/share/pixmaps/media-player-widget/play.png");
+			    PLAY_ICON);
   }
 }
 
@@ -94,10 +135,9 @@ _playlist_item_metadata_cb(MafwPlaylist *pls,
   GError* error = NULL;
   GValue* value;
   const gchar* title;
-  
   value = mafw_metadata_first(metadata, MAFW_METADATA_KEY_TITLE);
   title = g_value_get_string(value);
-  g_warning("title %s \n", title);
+
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
   if(title != NULL)
   {
@@ -117,7 +157,7 @@ _playlist_item_metadata_cb(MafwPlaylist *pls,
     }
     desktop_plugin->priv->artist = g_strdup(title);
   }
-  value = mafw_metadata_first(metadata, MAFW_METADATA_KEY_ARTIST);
+  value = mafw_metadata_first(metadata, MAFW_METADATA_KEY_ALBUM);
   title = g_value_get_string(value);
   if(title != NULL)
   {
@@ -127,52 +167,44 @@ _playlist_item_metadata_cb(MafwPlaylist *pls,
     }
     desktop_plugin->priv->album = g_strdup(title);
   }
-  gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
-}
 
-static void
-_on_media_changed_cb(MafwRenderer* mafw_renderer, gint index, gchar* object_id, gpointer user_data)
-{
-  g_warning("media_changed %s \n", object_id);
-  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
-  if(desktop_plugin->priv->mafw_playlist)
+  value = mafw_metadata_first(metadata, MAFW_METADATA_KEY_ALBUM_ART_SMALL_URI);
+  if(value != NULL)
   {
-    const gchar *const *keys;
-    keys = MAFW_SOURCE_LIST(MAFW_METADATA_KEY_TITLE,
-			    MAFW_METADATA_KEY_ARTIST,
-			    MAFW_METADATA_KEY_ALBUM);
-  
-    mafw_playlist_get_items_md(desktop_plugin->priv->mafw_playlist,
-			       index, index,
-			       keys,
-			       _playlist_item_metadata_cb,
-			       desktop_plugin, 
-			       NULL);
-  }  
-  //  gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
-}
-
-static void
-_on_metadata_changed_cb(MafwRenderer* mafw_renderer, gchar *name, GValueArray *value, gpointer user_data)
-{
-  g_warning("metadata_changed %s \n", name);
-}
-
-static void
-_on_state_changed_cb(MafwRenderer* mafw_renderer, gint state, gpointer user_data)
-{
-  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
-  g_warning("state %d\n", state);
-  desktop_plugin->priv->state = state;
-  _set_image_for_state(state, MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data));
-}
-
-static void
-_on_error_cb(MafwExtension* mafw_extension, guint domain, gint code,
-	     gchar *message,
-	     gpointer user_data)
-{
-  g_warning("error message %s\n", message);
+    title = g_value_get_string(value);
+    gchar* filename = NULL;
+    if(title != NULL && (filename = g_filename_from_uri(title,
+						       NULL,
+							NULL)) != NULL)
+    {
+      GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file_at_size(filename,
+							   64, 64, NULL);
+      if(pixbuf)
+      {
+	
+	gtk_image_set_from_pixbuf(GTK_IMAGE(desktop_plugin->priv->album_icon),
+				  pixbuf);
+	g_object_unref(pixbuf);
+	if(desktop_plugin->priv->album_pixbuf)
+	{
+	  g_object_unref(desktop_plugin->priv->album_pixbuf);
+	  desktop_plugin->priv->album_pixbuf = NULL;
+	}
+      }
+      g_free(filename);
+    }
+  }
+  else
+  {
+    if(desktop_plugin->priv->album_pixbuf == NULL)
+    {
+      desktop_plugin->priv->album_pixbuf = gdk_pixbuf_new_from_file_at_size(DEFAULT_ALBUM_ICON,
+									    64, 64, NULL);
+      gtk_image_set_from_pixbuf(GTK_IMAGE(desktop_plugin->priv->album_icon),
+				desktop_plugin->priv->album_pixbuf);
+    }
+  }
+  gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));
 }
 
 static void
@@ -185,7 +217,6 @@ _on_renderer_get_status_cb(MafwRenderer* mafw_renderer,
 			   const GError* error)
 {
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
-  desktop_plugin->priv->mafw_playlist = playlist;
   if(error)
   {
     g_warning("error on playbackstate cb %s\n", error->message);
@@ -193,8 +224,8 @@ _on_renderer_get_status_cb(MafwRenderer* mafw_renderer,
   const gchar *const *keys;
   keys = MAFW_SOURCE_LIST(MAFW_METADATA_KEY_TITLE,
 			  MAFW_METADATA_KEY_ARTIST,
-			  MAFW_METADATA_KEY_ALBUM);
-  
+			  MAFW_METADATA_KEY_ALBUM,
+			  MAFW_METADATA_KEY_ALBUM_ART_SMALL_URI);
   mafw_playlist_get_items_md(playlist,
 			     index, index,
 			     keys,
@@ -206,42 +237,46 @@ _on_renderer_get_status_cb(MafwRenderer* mafw_renderer,
 }
 
 static void
-_on_playlist_changed_cb(MafwRenderer* mafw_renderer, GObject* playlist, gpointer user_data)
+_on_media_changed_cb(MafwRenderer* mafw_renderer, gint index, gchar* object_id, gpointer user_data)
 {
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
-  desktop_plugin->priv->mafw_playlist = MAFW_PLAYLIST(playlist);
   mafw_renderer_get_status(desktop_plugin->priv->mafw_renderer,
 			   _on_renderer_get_status_cb, 
 			   desktop_plugin);
 }
 
 static void
-_connect_signal_handler(MafwRenderer* mafw_renderer, MediaPlayerWidgetHomePlugin* desktop_plugin)
+_on_state_changed_cb(MafwRenderer* mafw_renderer, gint state, gpointer user_data)
 {
-  desktop_plugin->priv->on_media_changed_handler_id = g_signal_connect(mafw_renderer,
-								       "media-changed",
-								       G_CALLBACK (_on_media_changed_cb),
-								       desktop_plugin);
-  desktop_plugin->priv->on_state_changed_handler_id = g_signal_connect(mafw_renderer,
-								       "state-changed",
-								       G_CALLBACK (_on_state_changed_cb),
-								       desktop_plugin);
-  desktop_plugin->priv->on_metadata_changed_handler_id = g_signal_connect(mafw_renderer,
-									  "metadata-changed",
-									  G_CALLBACK (_on_metadata_changed_cb),
-									  desktop_plugin);
-  desktop_plugin->priv->on_playlist_changed_handler_id = g_signal_connect(mafw_renderer,
-									  "playlist-changed",
-									  G_CALLBACK (_on_playlist_changed_cb),
-									  desktop_plugin);
-  desktop_plugin->priv->on_error_handler_id = g_signal_connect(mafw_renderer,
-							       "error",
-							       G_CALLBACK (_on_error_cb),
-							       desktop_plugin);
+  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  desktop_plugin->priv->state = state;
+  _set_image_for_state(state, MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data));
 }
 
 static void
-_disconnect_signal_handler(MafwRenderer* mafw_renderer, MediaPlayerWidgetHomePlugin* desktop_plugin)
+_on_metadata_changed_cb(MafwRenderer* mafw_renderer, gchar *name, GValueArray *value, gpointer user_data)
+{
+}
+
+static void
+_on_playlist_changed_cb(MafwRenderer* mafw_renderer, GObject* playlist, gpointer user_data)
+{
+  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  mafw_renderer_get_status(desktop_plugin->priv->mafw_renderer,
+			   _on_renderer_get_status_cb, 
+			   desktop_plugin);
+}
+
+static void
+_on_error_cb(MafwExtension* mafw_extension, guint domain, gint code,
+	     gchar *message,
+	     gpointer user_data)
+{
+  g_warning("error message %s\n", message);
+}
+
+static void
+_disconnect_renderer_signal_handler(MafwRenderer* mafw_renderer, MediaPlayerWidgetHomePlugin* desktop_plugin)
 {
   if(desktop_plugin->priv->on_media_changed_handler_id)
   {
@@ -251,13 +286,13 @@ _disconnect_signal_handler(MafwRenderer* mafw_renderer, MediaPlayerWidgetHomePlu
   }
   if(desktop_plugin->priv->on_state_changed_handler_id)
   {
-    g_signal_handler_disconnect (mafw_renderer,
+    g_signal_handler_disconnect(mafw_renderer,
 				 desktop_plugin->priv->on_state_changed_handler_id);
     desktop_plugin->priv->on_state_changed_handler_id = 0;
   }
   if(desktop_plugin->priv->on_metadata_changed_handler_id)
   {
-    g_signal_handler_disconnect (mafw_renderer,
+    g_signal_handler_disconnect(mafw_renderer,
 				 desktop_plugin->priv->on_metadata_changed_handler_id);
     desktop_plugin->priv->on_metadata_changed_handler_id = 0;
   }
@@ -267,12 +302,104 @@ _disconnect_signal_handler(MafwRenderer* mafw_renderer, MediaPlayerWidgetHomePlu
 				desktop_plugin->priv->on_playlist_changed_handler_id);
     desktop_plugin->priv->on_playlist_changed_handler_id = 0;
   }
+  /*
   if(desktop_plugin->priv->on_error_handler_id)
   {
-    g_signal_handler_disconnect (mafw_renderer,
+    g_signal_handler_disconnect(mafw_renderer,
 				 desktop_plugin->priv->on_error_handler_id);
     desktop_plugin->priv->on_error_handler_id = 0;
   }
+  */
+}
+
+static void
+_connect_renderer_signal_handler(MafwRenderer* mafw_renderer, MediaPlayerWidgetHomePlugin* desktop_plugin)
+{
+  // renderer signals
+  desktop_plugin->priv->on_media_changed_handler_id = g_signal_connect(mafw_renderer,
+								       "media-changed",
+								       G_CALLBACK (_on_media_changed_cb),
+								       desktop_plugin);
+
+  desktop_plugin->priv->on_state_changed_handler_id = g_signal_connect(mafw_renderer,
+								       "state-changed",
+								       G_CALLBACK (_on_state_changed_cb),
+								       desktop_plugin);
+
+  desktop_plugin->priv->on_metadata_changed_handler_id = g_signal_connect(mafw_renderer,
+									  "metadata-changed",
+									  G_CALLBACK (_on_metadata_changed_cb),
+									  desktop_plugin);
+
+  desktop_plugin->priv->on_playlist_changed_handler_id = g_signal_connect(mafw_renderer,
+									  "playlist-changed",
+									  G_CALLBACK (_on_playlist_changed_cb),
+									  desktop_plugin);
+  /*
+  // extension signals
+  desktop_plugin->priv->on_error_handler_id = g_signal_connect(mafw_renderer,
+							       "error",
+							       G_CALLBACK (_on_error_cb),
+							       desktop_plugin);
+  */
+}
+
+static gboolean 
+_init_renderer_playlist(gpointer user_data)
+{
+  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  desktop_plugin->priv->mafw_playlist_manager = mafw_playlist_manager_get();
+  MafwProxyPlaylist* playlist = mafw_playlist_manager_create_playlist(desktop_plugin->priv->mafw_playlist_manager,
+								      "FmpAudioPlaylist", NULL);
+  const gchar *const *keys;
+  keys = MAFW_SOURCE_LIST(MAFW_METADATA_KEY_TITLE,
+			  MAFW_METADATA_KEY_ARTIST,
+			  MAFW_METADATA_KEY_ALBUM,
+			  MAFW_METADATA_KEY_ALBUM_ART_SMALL_URI);
+  int last_playing_song = _mediaplayer_last_playlist_index();
+  mafw_playlist_get_items_md(MAFW_PLAYLIST(playlist),
+			     last_playing_song, last_playing_song,
+			     keys,
+			     _playlist_item_metadata_cb,
+			     desktop_plugin, 
+			     NULL);
+  return FALSE;
+}
+
+static void
+_on_renderer_get_status_cb_init(MafwRenderer* mafw_renderer,
+				MafwPlaylist* playlist,
+				guint index,
+				MafwPlayState state,
+				const gchar* object_id,
+				gpointer user_data,
+				const GError* error)
+{
+  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  if(error)
+  {
+    g_warning("error on playbackstate cb %s\n", error->message);
+  }
+  if(playlist == NULL)
+  {
+    g_timeout_add(2000,
+		  _init_renderer_playlist,
+		  desktop_plugin);
+    return;
+  }
+  const gchar *const *keys;
+  keys = MAFW_SOURCE_LIST(MAFW_METADATA_KEY_TITLE,
+			  MAFW_METADATA_KEY_ARTIST,
+			  MAFW_METADATA_KEY_ALBUM,
+			  MAFW_METADATA_KEY_ALBUM_ART_SMALL_URI);
+  mafw_playlist_get_items_md(playlist,
+			     index, index,
+			     keys,
+			     _playlist_item_metadata_cb,
+			     desktop_plugin, 
+			     NULL);
+  desktop_plugin->priv->state = state;
+  _set_image_for_state(state, MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data));
 }
 
 static void
@@ -285,12 +412,12 @@ _on_renderer_added(MafwRegistry* mafw_registry,
   {
     MafwRenderer* mafw_renderer = MAFW_RENDERER(renderer);
     desktop_plugin->priv->mafw_renderer = mafw_renderer;
-    _connect_signal_handler(mafw_renderer, desktop_plugin);
-    mafw_renderer_get_status(mafw_renderer,
-			     _on_renderer_get_status_cb, 
+    g_object_ref(mafw_renderer);
+    _connect_renderer_signal_handler(mafw_renderer, desktop_plugin);
+    mafw_renderer_get_status(desktop_plugin->priv->mafw_renderer,
+			     _on_renderer_get_status_cb_init, 
 			     desktop_plugin);
   }
-  g_warning("renderer added %s\n", mafw_extension_get_name(MAFW_EXTENSION(renderer)));
 }
 
 static void
@@ -299,8 +426,45 @@ _on_renderer_removed(MafwRegistry* mafw_registry,
 		     gpointer user_data)
 {
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
-  _disconnect_signal_handler(MAFW_RENDERER(renderer), desktop_plugin);
-  desktop_plugin->priv->mafw_renderer = NULL;
+  if(g_strcmp0(mafw_extension_get_name(MAFW_EXTENSION(renderer)), MEDIAPLAYER_RENDERER) == 0)
+  {
+    if(desktop_plugin->priv->mafw_renderer)
+    {
+      _disconnect_renderer_signal_handler(desktop_plugin->priv->mafw_renderer, desktop_plugin);
+      g_object_unref(desktop_plugin->priv->mafw_renderer);
+      desktop_plugin->priv->mafw_renderer = NULL;
+    }
+  }
+}
+
+static void
+_connect_registry_signal_handler(MafwRegistry* mafw_registry, MediaPlayerWidgetHomePlugin* desktop_plugin)
+{
+  desktop_plugin->priv->on_renderer_added_id = g_signal_connect(mafw_registry, 
+								"renderer_added", 
+								G_CALLBACK(_on_renderer_added), 
+								desktop_plugin);
+
+  desktop_plugin->priv->on_renderer_removed_id = g_signal_connect(mafw_registry, 
+								  "renderer_removed", 
+								  G_CALLBACK(_on_renderer_removed), 
+								  desktop_plugin);
+}
+
+static void
+_disconnect_registry_signal_handler(MafwRegistry* mafw_registry, MediaPlayerWidgetHomePlugin* desktop_plugin)
+{
+  if(desktop_plugin->priv->on_renderer_added_id != 0)
+  {
+    g_signal_handler_disconnect(mafw_registry,
+				desktop_plugin->priv->on_renderer_added_id);
+    desktop_plugin->priv->on_renderer_added_id = 0;
+  }
+  if(desktop_plugin->priv->on_renderer_removed_id != 0)
+  {
+    g_signal_handler_disconnect(mafw_registry, desktop_plugin->priv->on_renderer_removed_id);
+    desktop_plugin->priv->on_renderer_removed_id = 0;
+  }
 }
 
 static void
@@ -313,8 +477,9 @@ _init_mafw(MediaPlayerWidgetHomePlugin* desktop_plugin)
   if(mafw_registry == NULL)
   {
     g_warning("can not init mafw registry\n");
+    return;
   }
-  
+  desktop_plugin->priv->mafw_registry = mafw_registry;
   mafw_shared_init(mafw_registry, &error);
   if(error)
   {
@@ -329,25 +494,20 @@ _init_mafw(MediaPlayerWidgetHomePlugin* desktop_plugin)
     while(renderer)
     {
       MafwRenderer* mafw_renderer = MAFW_RENDERER(renderer->data);
-      g_warning("renderer %s\n", mafw_extension_get_name(MAFW_EXTENSION(mafw_renderer)));
       if(g_strcmp0(mafw_extension_get_name(MAFW_EXTENSION(mafw_renderer)), MEDIAPLAYER_RENDERER) == 0)
       {
 	desktop_plugin->priv->mafw_renderer = mafw_renderer;
-	_connect_signal_handler(mafw_renderer, desktop_plugin);
-	mafw_renderer_get_status(mafw_renderer,
+	g_object_ref(mafw_renderer);
+	_connect_renderer_signal_handler(mafw_renderer, desktop_plugin);
+	mafw_renderer_get_status(desktop_plugin->priv->mafw_renderer,
 				 _on_renderer_get_status_cb, 
 				 desktop_plugin);
       }
       renderer = renderer->next;
     }
   }
-  else
-  {
-    g_warning("no renderers\n");
-  }
-  g_signal_connect(mafw_registry, "renderer_added", G_CALLBACK(_on_renderer_added), desktop_plugin);
-  g_signal_connect(mafw_registry, "renderer_removed", G_CALLBACK(_on_renderer_removed), desktop_plugin);
-  //  g_signal_connect(mafw_registry, "source_added", G_CALLBACK(source_added_cb), self);
+  _connect_registry_signal_handler(mafw_registry, desktop_plugin);
+  //  g_signal_connect(mafw_registry, "source_added", G_CALLBACK(source_added_cb), desktop_plugin);
   //  g_signal_connect(mafw_registry, "source_removed", G_CALLBACK(source_removed_cb), NULL);
 }
 
@@ -358,35 +518,10 @@ media_player_widget_realize(GtkWidget* widget)
   GdkScreen *screen = gtk_widget_get_screen(widget);
   gtk_widget_set_colormap(widget, gdk_screen_get_rgba_colormap(screen));
   gtk_widget_set_app_paintable(widget, TRUE);
-  //  desktop_plugin->priv->iD = hd_home_plugin_item_get_applet_id(HD_HOME_PLUGIN_ITEM(widget));
   _init_mafw(desktop_plugin);
   GTK_WIDGET_CLASS(media_player_widget_home_plugin_parent_class)->realize(widget);
 }
 
-static void
-media_player_widget_finalize(GObject* object)
-{
-  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(object);
-  if(desktop_plugin->priv->mafw_renderer)
-    _disconnect_signal_handler(desktop_plugin->priv->mafw_renderer, desktop_plugin);
-  if(desktop_plugin->priv->title)
-  {
-    g_free(desktop_plugin->priv->title);
-  }
-  if(desktop_plugin->priv->album)
-  {
-    g_free(desktop_plugin->priv->album);
-  }
-  if(desktop_plugin->priv->artist)
-  {
-    g_free(desktop_plugin->priv->artist);
-  }
-  if(desktop_plugin->priv->background_pixbuf)
-    g_object_unref(desktop_plugin->priv->background_pixbuf);
-  //  g_free(desktop_plugin->priv->iD);
-  G_OBJECT_CLASS(media_player_widget_home_plugin_parent_class)->finalize(object);
-}
-
 static void
 _on_current_desktop(GtkWidget* widget,
 		    gpointer data)
@@ -405,6 +540,7 @@ _on_playback_state_changed(MafwRenderer* renderer,
 			   gpointer user_data,
 			   const GError* error)
 {
+  g_warning("playback state changed\n");
   if(error)
   {
     g_warning("error on playbackstate cb %s\n", error->message);
@@ -427,8 +563,8 @@ _on_backward_button_press(GtkWidget* widget, GdkEventButton* event, gpointer use
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
   if(desktop_plugin->priv->mafw_renderer)
   {
-    mafw_renderer_next(desktop_plugin->priv->mafw_renderer,
-		       _on_playback_state_changed, desktop_plugin);
+    mafw_renderer_previous(desktop_plugin->priv->mafw_renderer,
+			   _on_playback_state_changed, desktop_plugin);
   }
 }
 
@@ -436,8 +572,8 @@ static void
 _on_forward_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
 {
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
-  mafw_renderer_previous(desktop_plugin->priv->mafw_renderer,
-			 _on_playback_state_changed, desktop_plugin);
+  mafw_renderer_next(desktop_plugin->priv->mafw_renderer,
+		     _on_playback_state_changed, desktop_plugin);
 }
 
 static void
@@ -461,17 +597,99 @@ _on_play_pause_button_press(GtkWidget* widget, GdkEventButton* event, gpointer u
 			 _on_playback_state_changed, desktop_plugin);
 
   }
-  else
+  else if(desktop_plugin->priv->state == -1)
   {
-    g_warning("current state %d\n", desktop_plugin->priv->state);
+    desktop_plugin->priv->mafw_playlist_manager = mafw_playlist_manager_get();
+    MafwProxyPlaylist* playlist = mafw_playlist_manager_create_playlist(desktop_plugin->priv->mafw_playlist_manager,
+									"FmpAudioPlaylist", NULL);
+    if(mafw_renderer_assign_playlist(desktop_plugin->priv->mafw_renderer,
+				     MAFW_PLAYLIST(playlist),
+				     NULL))
+    {
+      int index = _mediaplayer_last_playlist_index();
+      mafw_renderer_play(desktop_plugin->priv->mafw_renderer,
+			 _on_playback_state_changed, desktop_plugin);
+      mafw_renderer_goto_index(desktop_plugin->priv->mafw_renderer,
+			       index,
+			       NULL, NULL);
+    }
   }
 }
 
 static void
-_on_album_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
+_on_label_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
 {
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  GtkWidget* dialog = hildon_picker_dialog_new(GTK_WINDOW(desktop_plugin));
+  GtkWidget* selector = hildon_touch_selector_new_text();
+  hildon_touch_selector_set_column_selection_mode(HILDON_TOUCH_SELECTOR(selector),
+						  HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
+  desktop_plugin->priv->mafw_playlist_manager = mafw_playlist_manager_get();
+  GArray* playlists = mafw_playlist_manager_list_playlists(desktop_plugin->priv->mafw_playlist_manager, NULL);
+  int i = 0;
+  for(i = 0; i< playlists->len;++i)
+  {
+    MafwPlaylistManagerItem* item = &g_array_index(playlists, MafwPlaylistManagerItem, i);
 
+    if(g_strstr_len(item->name, 3, "Fmp") == NULL)
+    {
+      hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR(selector),
+					item->name);
+    }
+  }
+  hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR(selector),
+				   0, -1);
+  hildon_picker_dialog_set_selector(HILDON_PICKER_DIALOG(dialog), 
+				    HILDON_TOUCH_SELECTOR(selector));
+  gtk_widget_show_all(dialog);
+  MafwProxyPlaylist* new_playlist = NULL;
+  int response = gtk_dialog_run(GTK_DIALOG(dialog));
+  if(response == GTK_RESPONSE_OK)
+  {
+    gint index = hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(selector), 0);
+    gchar* selected = hildon_touch_selector_get_current_text(HILDON_TOUCH_SELECTOR(selector));
+    new_playlist= mafw_playlist_manager_create_playlist(desktop_plugin->priv->mafw_playlist_manager, 
+							selected,
+							NULL);
+      
+    MafwProxyPlaylist* old_playlist= mafw_playlist_manager_create_playlist(desktop_plugin->priv->mafw_playlist_manager, 
+									   "FmpAudioPlaylist",
+									   NULL);
+      
+      
+    if(mafw_playlist_clear(MAFW_PLAYLIST(old_playlist),
+			   NULL))
+    {
+      guint playlist_size = mafw_playlist_get_size(MAFW_PLAYLIST(new_playlist), NULL);
+      int i;
+      gchar** items = mafw_playlist_get_items(MAFW_PLAYLIST(new_playlist),
+					      0,
+					      playlist_size-1, 
+					      NULL);
+      gchar** items2 = items;
+      for(i=0;i<playlist_size;++i)
+      {
+	GError* error = NULL;
+	mafw_playlist_append_item(MAFW_PLAYLIST(old_playlist),
+				  (*items2),
+				  &error);
+	items2++;
+	if(error)
+	{
+	  g_error_free(error);
+	}
+      }
+      g_strfreev(items);
+    }
+    if(mafw_renderer_assign_playlist(desktop_plugin->priv->mafw_renderer,
+				     MAFW_PLAYLIST(old_playlist),
+				     NULL))
+    {}
+  }
+  mafw_playlist_manager_free_list_of_playlists(playlists);
+  gtk_widget_destroy(dialog);
+  mafw_renderer_play(desktop_plugin->priv->mafw_renderer,
+		     _on_playback_state_changed, desktop_plugin);
 }
 
 static gboolean
@@ -481,37 +699,40 @@ _on_label_expose(GtkWidget* widget, GdkEventExpose* event, gpointer user_data)
   cairo_t *cr;
   cr = gdk_cairo_create(GDK_DRAWABLE(widget->window));
   gdk_cairo_region(cr, event->region);
-  cairo_translate(cr, 64,0);
   cairo_clip(cr);
+  cairo_translate(cr,event->area.x,event->area.y);
   cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
   gdk_cairo_set_source_pixbuf(cr, desktop_plugin->priv->background_pixbuf, 0, 0);
   cairo_paint(cr);
   PangoLayout* layout;
   PangoFontDescription* desc;
   layout = pango_cairo_create_layout(cr);
-  g_warning("xpose %s\n", desktop_plugin->priv->title);
+  desc = pango_font_description_from_string("Sans Bold 12");
+  pango_layout_set_font_description(layout ,desc);
+  pango_layout_set_width(layout, (event->area.width-5)*1024);
+  pango_layout_set_ellipsize(layout, PANGO_ELLIPSIZE_END);
+
+  pango_font_description_free(desc);
+  cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0); 
+  cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+
   if(desktop_plugin->priv->title)
     pango_layout_set_text(layout, desktop_plugin->priv->title, -1);
   else
     pango_layout_set_text(layout, "", -1);
-  desc = pango_font_description_from_string("Sans Bold 8");
-  pango_layout_set_font_description(layout ,desc);
-  pango_font_description_free(desc);
-  cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0); 
-  cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-  cairo_move_to(cr, 0, 32);
+  cairo_move_to(cr, 5, 8);
   pango_cairo_show_layout(cr, layout);
   if(desktop_plugin->priv->artist)
     pango_layout_set_text(layout, desktop_plugin->priv->artist, -1);
   else
     pango_layout_set_text(layout, "", -1);
-  cairo_move_to(cr, 0, 8);
+  cairo_move_to(cr, 5, 24);
   pango_cairo_show_layout(cr, layout);
   if(desktop_plugin->priv->album)
     pango_layout_set_text(layout, desktop_plugin->priv->album, -1);
   else
     pango_layout_set_text(layout, "", -1);
-  cairo_move_to(cr, 0, 8);
+  cairo_move_to(cr, 5, 40);
   pango_cairo_show_layout(cr, layout);
   g_object_unref(layout);
   cairo_destroy(cr);
@@ -520,10 +741,27 @@ _on_label_expose(GtkWidget* widget, GdkEventExpose* event, gpointer user_data)
 }
 
 static void
-_on_label_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
+_on_album_button_press(GtkWidget* widget, GdkEventButton* event, gpointer user_data)
 {
   MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(user_data);
+  DBusGConnection* dbus_conn = hd_home_plugin_item_get_dbus_g_connection(&desktop_plugin->hitem,
+                                                                         DBUS_BUS_SESSION,
+                                                                         NULL);
+  DBusGProxy* dbus_proxy = dbus_g_proxy_new_for_name(dbus_conn,
+                                                     HD_APP_MGR_DBUS_NAME,
+                                                     HD_APP_MGR_DBUS_PATH,
+                                                     HD_APP_MGR_DBUS_IFACE);
+  if(dbus_proxy)
+  {
+    dbus_g_proxy_call_no_reply(dbus_proxy,
+                               HD_APP_MGR_LAUNCH_APPLICATION,
+                               G_TYPE_STRING,
+                               MEDIAPLAYER_APP_NAME,
+                               G_TYPE_INVALID,
+                               G_TYPE_INVALID);
 
+    g_object_unref(dbus_proxy);
+  }
 }
 
 static void
@@ -552,26 +790,25 @@ _init_ui(MediaPlayerWidgetHomePlugin* desktop_plugin)
   GtkWidget* label = gtk_event_box_new();
   gtk_event_box_set_visible_window(GTK_EVENT_BOX(label), FALSE); 
   gtk_container_set_border_width(GTK_CONTAINER(label), 0);
-  g_signal_connect(GTK_CONTAINER(album), "button-release-event", G_CALLBACK(_on_label_button_press), 
+  g_signal_connect(GTK_CONTAINER(label), "button-release-event", G_CALLBACK(_on_label_button_press), 
 		   desktop_plugin);
-  GtkWidget* backward_icon = gtk_image_new_from_file("/usr/share/pixmaps/media-player-widget/backward.png");
-  GtkWidget* forward_icon = gtk_image_new_from_file("/usr/share/pixmaps/media-player-widget/forward.png");
-  desktop_plugin->priv->label_background = gtk_image_new_from_file("/usr/share/pixmaps/media-player-widget/label_background.png");
+  GtkWidget* backward_icon = gtk_image_new_from_file(BACKWARD_ICON);
+  GtkWidget* forward_icon = gtk_image_new_from_file(FORWARD_ICON);
+  desktop_plugin->priv->label_background = gtk_image_new_from_file(LABEL_BACKGROUND_ICON);
   g_signal_connect(desktop_plugin->priv->label_background, "expose-event", G_CALLBACK(_on_label_expose), 
 		   desktop_plugin);
-  desktop_plugin->priv->background_pixbuf = gdk_pixbuf_new_from_file("/usr/share/pixmaps/media-player-widget/label_background.png", NULL);
-  desktop_plugin->priv->play_pause_icon = gtk_image_new_from_file("/usr/share/pixmaps/media-player-widget/pause.png");
-  desktop_plugin->priv->album_icon = gtk_image_new_from_file("/usr/share/pixmaps/media-player-widget/player.png");
-  g_warning("build container\n");
+  desktop_plugin->priv->background_pixbuf = gdk_pixbuf_new_from_file(LABEL_BACKGROUND_ICON, NULL);
+  desktop_plugin->priv->play_pause_icon = gtk_image_new_from_file(PLAY_ICON);
+  desktop_plugin->priv->album_pixbuf = gdk_pixbuf_new_from_file_at_size(DEFAULT_ALBUM_ICON,
+									64, 64, NULL);
+  desktop_plugin->priv->album_icon = gtk_image_new_from_pixbuf(desktop_plugin->priv->album_pixbuf);
+
   gtk_container_add(GTK_CONTAINER(backward), backward_icon);
   gtk_container_add(GTK_CONTAINER(forward), forward_icon);
   gtk_container_add(GTK_CONTAINER(play_pause), desktop_plugin->priv->play_pause_icon);
   gtk_container_add(GTK_CONTAINER(album), desktop_plugin->priv->album_icon);
   gtk_container_add(GTK_CONTAINER(label), desktop_plugin->priv->label_background);
-  gtk_widget_set_size_request(label,
-			      206,
-			      64);
-  g_warning("build container done\n");
+  gtk_widget_set_size_request(label, 206, 64);
   GtkWidget* label_box = gtk_hbox_new(FALSE, 0);
   GtkWidget* button_box = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(button_box), backward, TRUE, TRUE, 0); 
@@ -590,29 +827,84 @@ static void
 media_player_widget_home_plugin_init(MediaPlayerWidgetHomePlugin* desktop_plugin)
 {
   desktop_plugin->priv = MEDIA_PLAYER_WIDGET_HOME_PLUGIN_GET_PRIVATE(desktop_plugin);
+  desktop_plugin->priv->mafw_registry = NULL;
   desktop_plugin->priv->mafw_renderer = NULL;
   desktop_plugin->priv->mafw_playlist = NULL;
+  desktop_plugin->priv->mafw_playlist_manager = NULL;
+  desktop_plugin->priv->album_pixbuf = NULL;
+  desktop_plugin->priv->state = -1;
+  desktop_plugin->priv->title = NULL;
+  desktop_plugin->priv->artist = NULL;
+  desktop_plugin->priv->album = NULL;
   desktop_plugin->priv->on_media_changed_handler_id = 0;
   desktop_plugin->priv->on_state_changed_handler_id = 0;
   desktop_plugin->priv->on_metadata_changed_handler_id = 0;
   desktop_plugin->priv->on_playlist_changed_handler_id = 0;
   desktop_plugin->priv->on_error_handler_id = 0;
-  desktop_plugin->priv->title = NULL;
-  desktop_plugin->priv->artist = NULL;
-  desktop_plugin->priv->album = NULL;
+  desktop_plugin->priv->on_renderer_added_id = 0;
+  desktop_plugin->priv->on_renderer_removed_id = 0;
   _init_ui(desktop_plugin);
   //  hd_home_plugin_item_set_settings(HD_HOME_PLUGIN_ITEM (desktop_plugin), TRUE);
   //  g_signal_connect(desktop_plugin, "show-settings", G_CALLBACK(_show_settings_dialog), NULL);
   g_signal_connect(desktop_plugin, "notify::is-on-current-desktop", G_CALLBACK(_on_current_desktop), NULL);
 }
 
+static void
+media_player_widget_finalize(GObject* object)
+{
+  MediaPlayerWidgetHomePlugin* desktop_plugin = MEDIA_PLAYER_WIDGET_HOME_PLUGIN(object);
+  if(desktop_plugin->priv->mafw_renderer)
+  {
+    _disconnect_renderer_signal_handler(desktop_plugin->priv->mafw_renderer, desktop_plugin);
+    g_object_unref(desktop_plugin->priv->mafw_renderer);
+    desktop_plugin->priv->mafw_renderer = NULL;
+  }
+  if(desktop_plugin->priv->mafw_registry)
+  {
+    _disconnect_registry_signal_handler(desktop_plugin->priv->mafw_registry, desktop_plugin);
+  }
+
+  if(desktop_plugin->priv->title)
+  {
+    g_free(desktop_plugin->priv->title);
+    desktop_plugin->priv->title = NULL;
+  }
+  if(desktop_plugin->priv->album)
+  {
+    g_free(desktop_plugin->priv->album);
+    desktop_plugin->priv->album = NULL;
+  }
+  if(desktop_plugin->priv->artist)
+  {
+    g_free(desktop_plugin->priv->artist);
+    desktop_plugin->priv->artist = NULL;
+  }
+  if(desktop_plugin->priv->background_pixbuf)
+  {
+    g_object_unref(desktop_plugin->priv->background_pixbuf);
+    desktop_plugin->priv->background_pixbuf = NULL;
+  }
+  if(desktop_plugin->priv->album_pixbuf)
+  {
+    g_object_unref(desktop_plugin->priv->album_pixbuf);
+    desktop_plugin->priv->album_pixbuf = NULL;
+  }
+  G_OBJECT_CLASS(media_player_widget_home_plugin_parent_class)->finalize(object);
+}
+
+const gchar *
+g_module_check_init (GModule *module)
+{
+     g_module_make_resident(module);
+     return NULL;
+}
+
 static void
 media_player_widget_home_plugin_class_init(MediaPlayerWidgetHomePluginClass* klass)
 {
   GtkWidgetClass* widget_class = GTK_WIDGET_CLASS(klass);
   g_type_class_add_private(klass, sizeof(MediaPlayerWidgetHomePluginPrivate));
   widget_class->realize = media_player_widget_realize;
-  //  widget_class->expose_event = media_player_widget_expose;
   G_OBJECT_CLASS(klass)->finalize = media_player_widget_finalize;
 }
 
